; INIT_MALAX_2D -> IN_MALAX_2D -> INITIAL_DATE
;   remplissent le common MALAX, utilise par CALCPOS pour passer des coordon-
;	nees interferometriques aux coordonees heliographiques.
;
;   INIT_MALAX_2D :
;    . recoit les coord de la source (h meridien, declinaison) et sa nature
;	(solaire ou non).
;    . leve les drapeaux de toutes les extensions : ie0, ie1, ie2, ins45.
;    . indique le choix soleil ou source
;    . baisse les drapeux de correction ionospherique icorion et icorpoi 
;	(au 18 sept 01)
;    . appelle IN_MALAX_2D.
;
;   IN_MALAX_2D :
;    . definit les parametres du reseau,
;    . calcule le numero du jour dans l'annee et les parametres solaire (hmer,
;	 delta, les derives horaires ghmer et gdelta, l'angle p). Ces parame-
;	 tres sont mis a zero sauf hmer pour une source non solaire.
;    . met tout cela dans le common MALAX.

; Voir commentaires dans CALCPOS au sujet du choix de nns (ayant provoque un
;   ecrasement du soleil en NS en sept 01).


;+ ***********************************************************************
; NAME:
;	INIT_MALAX_2D	
;
; PURPOSE:
;	Cette procedure initialise le COMMON MALAX utilise par CALCPOS
;	pour calculer les coordonnees heliographiques d'un point
;
; CATEGORY:
;	NRH General
;
; CALLING SEQUENCE:
;	INIT_MALAX_2D, date, hmer, dec, sref
;
; INPUTS:
;	DATE	Date en [Jour, Mois, An]
;	HMER	INTARR[3], heure de passage au meridien, utilise uniquement
;			pour les sources
;	DEC	INTARR[3}, declinaison utilise uniquement pour les sources
;	SREF	= 'SOLEIL' pour les soleil
;
; KEYWORD PARAMETERS:
;	Non
;
; OUTPUTS:
;	Non
;
; COMMON BLOCKS:
;	Non
;
; SIDE EFFECTS:
;	Ce module contient les procedures suivantes
;		INITIAL_DATE	conversion de date
;		IN_MALAX_2D	calcul d'initialisation
;
; RESTRICTIONS:
;     HMER et DEC doivent etre declares avant l'appel, meme dans le cas
;	du soleil. Les valeurs contnues dans ces 2 tableaux peuvent cependant
;	etre quelconques
;
; PROCEDURE:
;     En sortie le COMMON MALAX est initialise pour usage avec des images
;	interferometriques 128 * 128, obtenues avec E1, E2, E0, NS45 (dew=50m,
;	dns=54m).
;     Ces choix sont figes dans la courte procedure appelante INIT_MALAX_2D
;	mais ils ne sont pas necessaires a IN_MALAX qui est plus general et
;	pourrait etre appel'e pour l'ancien 2 * 1D ou les images EW et NS (sans
;	E0 ni NS45) sont definies sur 64 pts.
;
; EXAMPLE:
;	Please provide a simple example here
;
; MODIFICATION HISTORY:
;  Ecrit par: A Kerdraon
;  Adapte par J Bonmartin@obspm.fr, Claude.Mercier@obspm.fr
;  01 sept 10 : ie0, ie1 et ie2 (drapeaux d'utilisation des antrenne E0, E1 et 
;		  E2, qui permettent de calculer dew, dns, new, nns) sont defi-
;		  nis dans INIT_MALAX_RH_2D de facon a etre coherent avec les
;		  choix faits dans les procedures en aval de RH_DPNEW ->
;		  RH_DPATCHFITS_NRH.
;-*******************************************************************

;_____________________________________________________________________________


PRO INITIAL_DATE, ij, im, ian, n
    ; Suite de l'initialisation (obtention de la date)
    ; Fortement simplifie rapport a Mercier: juste calcul du jour et autres 
    ;  parametres, totalement retire les histoires de traitement de fichier.

    m = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
    if (ian gt 1980) then begin
	; on ne fait rien, la date est complete.
    endif else begin		; la date est exprimee sur 2 chiffres
	if (ian lt 50) then  ian = ian + 2000
	if (ian gt 50) then  ian = ian + 1900
	    ; ces rustines sont valables entre 1950 et 2050, date a laquelle on
	    ;   sera tous morts.
    endelse
	; Lignes sont reprises de RH_CORR_ANTENNES le 15 dec 04 qui remplacent
	;   le traitement ancien :
	;	if (ian lt 1900) then  ian = ian + 1900

; Calcul du numero de jour dans l'annee.
    n = 0
    if (im eq 1) then  goto, janvier
    im1 = im - 1
    for k = 0, im1-1 do n = n + m(k)

janvier:    n = n + ij		; n numero du jour dans l annee

    return
    end			; fin de INITIAL_DATE.

;______________________________________________________________________________

PRO IN_MALAX_2D, $			; tous les arguments sont en entree.
	ie0   ,  ie1     ,  ie2     ,  ins45 ,			$
	isour0,  rot_p   ,					$
	adh   ,  adm     ,  ads     ,  deldeg,  delmin,		$
	delsec,  icorion0,  icorpoi0,  ij0   ,  im0   ,  ian0
	

; But : 2eme maillon de l'initialisation du common MALAX, utilise par CALCPOS 
;	  pour passer des coordonnees interferometriques aux coordonees helio-
;	  graphiques.
;	La raison pour laquelle on utilise la cascade INIT_MALAX_2D -> IN_MALAX
;	  est expliquee dans INIT_MALAX_2D.
;	L'heure de passage du soleil au meridien est calculee, alors qu'elle
;	  est fournie dans le cas d'une observation solaire. C'est pour en
;	  disposer en vue de correction ionospheriques pour une source non
;	  solaire (ou ahmersol n'est pas fourni).
;	  

; Modifications :
; 01 jul 11  Ajout de ins45 dans la liste d'appel pour decider si nns=64 ou 128
; 02 mar 27  Ajout de rot_p pour correction angle p : 0 sans, 1 avec.
;    avr  3: Mise au clair de commentaires sur new, nns, ins45 etc. dans ce
;		fichier et dans CALCPOS. 

; MALAX en IDL. Premiere partie = initialisations
; On ne considere pas les effets de l'ionosphere dans un premier temps.
; Quelques arguments sont recopiees dans un common pour rester disponibles 
;  dans le module de calcul

; Projet de re-arrangement plus logique des variables du common
;   common MALAX, $
;	ij, im, ian, isour, $		; date et source (soleil ou autre).
;	icorion, icorpoi, $		; correction iono (et Poitiers)
;	etmer, edec, $			; corr temps merid et declin
;	dew, hew, u,  new, $		; param reseau EW.
;	dns, hns, sinl, cosl, nns, $	; param reseau NS. 
;	a1, $ 				; ??
;	c, $				; vitesse lumiere
;	ahmersol, ghmer, $		; temps merid soleil et derive
;	sind, cosd, gdel, $		; declinaison soleil et derive
;	sinp, cosp, rsol, $		; angle P et rayon angulaire solaire.
;	ahmer  

    common MALAX, $	; Sortie de IN_MALAX_2D (appele par INIT_MALAX_2D)
			; Procedures utilisant le common MALAX :
			;   . CALCPOS,
			;   . VISIB_CYG (dans AUTOCAL_SUB),
			;   . RH_GRILLE pour controles seulement
			;   . RH_MAILLE_3 (appel'e par RH_DPATCHFITS_NRH) qui
			;	calcule la maille.
			;   . le soft du 2 * 1D (lappel de IN_MALAX ne doit pas
			;	alors etre fait par INIT_MALAX_2D).
	ij, im, ian, icorpoi, icorion, isour, $
	etmer, edec, dew, new, hew, u, dns, hns, a1, nns, sinl, cosl, $
	sinp, cosp, gdel, ghmer, ahmer, c, rsol, ahmersol, sind, cosd

    ij      = ij0
    im      = im0
    ian     = ian0
    icorpoi = icorpoi0
    icorion = icorion0
    isour   = isour0 			; 1 pour solail, 2 pour source

    etmer =0.				; erreur temps merid dans le temps reel
    edec  =0.				; ------ declinaison ------------------
    if (isour eq 2) then begin		; source non solaire.
	ahmersour = adh + adm/60. + ads/3600.
	if (deldeg lt 0) then begin
		if (delmin gt 0) then  delmin = - delmin
		if (delsec gt 0) then  delsec = - delsec
	endif
	deltasour = !PI/180.* (deldeg + delmin/60. + delsec/3600.)
	sindsour  = sin(deltasour)
	cosdsour  = cos(deltasour)
    endif

; Parametres des reseau 
	; Les nbres de points new et nns definis ici sont ceux utilises par
	;   Malax et Pos_helio. Ce sont les puissances de 2 immediatement supe-
	;   rieures aux nombres de pts donnant l'echantillonnage minimum en EW 
	;   et NS des images (ces nbres de pts sont egaux a 2 fois le nbre
	;   d'harmoniques  + 1 (en pratique 2 fois)).
	; On rappelle que Malax et Pos_helio utilisent 
	;	r * c/freq / (dew*new)   et   s * c/freq / (dns*nns)
	;  Ce sont ainsi r / (dew*new)  et  s / (dns*nns) qui interviennent.
; Parametres utilises dans INIT_MALAX jusqu'au 17 fev 2004
;  Reseau EW
    if (ie0 eq 0) then begin
;	dew = 99.805
;	if (ie2 eq 0) then  new = 32
;	if (ie2 eq 1) then  new = 64
    endif
    if (ie0 eq 1) then begin
;	dew = 49.903
;;	if (ie2 eq 0) then  new=64
	if (ie2 eq 1) then  new=128
    endif
;    hew =  0.00230	; longitude reseau ew (radians).
;    u   = -0.00400	; inclinaison reseau ew sur l'equateur (radians).
;  Reseau NS
;    dns  = 54.274	; pas du reseau NS (metres)
;    hns  = 0.042383	; angle horaire du reseau NS (radians).
;    al   = 0.826431	; latitude du reseau NS (radians).
; Parametres utilises dans INIT_MALAX apres le 17 fev 2004, identiques a ceux 
	; envoyes le 13 oct 2003 a Andee Coffre pour la version du Temos Reel
	; avec les antennes AA.
    if (ie0 eq 0) then begin
	dew = 99.810
	if (ie2 eq 0) then  new = 32
	if (ie2 eq 1) then  new = 64
    endif
    if (ie0 eq 1) then begin
	dew = 49.905
	if (ie2 eq 0) then  new=64
	if (ie2 eq 1) then  new=128
    endif
    hew =  0.00248	; longitude reseau ew (radians).
    u   = -0.00380	; inclinaison reseau ew sur l'equateur (radians).
;  Reseau NS
    dns  = 54.274	; pas du reseau NS (metres)
    hns  = 0.042190	; angle horaire du reseau NS (radians).
    al   = 0.826540	; latitude du reseau NS (radians).


    sinl = sin(al)
    cosl = cos(al)
			; Rappel : ins45 est une variable interne INIT_MALAX_2D
			;   prise egale a 1. 
    if (ins45 eq 0) then  nns = 64	; nbre de points sur l'image NS. 
					; Rappel : Seul le produit dns*nns 
					; intervient. C'est le meme pour le NS
					; 169 MHz et le NS MF sans NS45.
    if (ins45 eq 1) then  nns = 128	; rajoute le 11 jul 01. Dorenavant
					;   new = nns = 128 de facon constante
					;   en aval de RH_DPNEW.

; calcul du jour dans l'annee
    INITIAL_DATE, ij, im, ian, n

; Initialisation ionosphere (a completer)
    c = 299.792500
    a = 2 * !pi / 365.25
    deltasol = !pi / 180. * 23.43 * sin(a * (n - 80))
    sindsol  = sin(deltasol)
    cosdsol  = cos(deltasol)
    rsol     = 16.020 + 0.268 * sin(a * (n - 278))	; ray sol (min d'arc).
    ahmersol = 11.854 + 0.125 * sin(a * n)  +  0.1617 * sin(2 * a * (n+9))

; Parametres de position et de derive de la source (solaire ou non) :
    if (isour eq 1) then  begin		; soleil.
;     Heure meridien et declinaison
	ahmer = ahmersol			; heures decimales (TU).
	delta = deltasol			; radians.
	cosd  = cosdsol
	sind  = sindsol
;     Angle p
	    ; Notations :
	    ;   At	axe polaire terrestre (oriente S -> N) projete sur le
	    ;							disque solaire.
	    ;   As	----------- solaire    --------------
	    ;   p = (At, As) compte dans le sens trigonometrique usuel.
	    ;        -p est l'angle dont il faut faire tourner l'image solaire
	    ;		pour rendre l'axe polaire solaire vertical.
	    ;         Inversement p est l'angle dont il faut faire tourner
	    ;		l'image heliographique pour que la projection de l'axe
	    ;		terrestre y soit verticale. On sait alors calculer
	    ;		les angles que font les lignes de lobes secondaire de
	    ;		l'EW et du NS avec l'axe polaire terrestre, en fonction
	    ;		de l'angle horaire et de la declinaison
	    ;  Rappel mnemotechnique : p est <0 de jan a jul, avec un extremum
	    ;		en mars.
	if (rot_p eq 0) then  p = 0
	if (rot_p eq 1) then  p = !pi/180.*26.32*sin(a*(n-188))
	sinp  = sin(p)
	cosp  = cos(p)
;     Taux de derive en angle horaire  du temps meridien et de la declinaison 
	    ; (en rad / rad)
	n = n + 1
	deltbsol = !pi / 180. * 23.43 * sin(a * (n - 80))
	bhmersol = 11.854 + 0.125 * sin(a * n) + 0.1617 *sin(2. * a *(n+9))
	gdel  = (deltbsol - deltasol) / (2. * !pi)
	ghmer = !pi / 12. * (bhmersol - ahmersol) / (2. * !pi)
    endif

    if (isour eq 2) then  begin		; source non solaire.
	ahmer = ahmersour 			; heures decimales (TS).
	delta = deltasour			; radians.
	cosd  = cosdsour
	sind  = sindsour
;     Mise a zero de l'angle P et des derives en angle horaire et declinaison 
	    ; (le calcul pour avoir les coordonnees angulaires normales est
	    ;  toujours fait dans POS_HELIO).
        cosp  = 1.
        sinp  = 0.
        ghmer = 0.
        gdel  = 0.
	print, ' '
	print, 'INIT_MALAX_2D :'
	print,'      Source non solaire. angle P et derives prises egales a 0'
	deldeg = 180/!pi * delta
;	print,'Source non solaire. Valeurs dans init_malax:'
;	print, ' delta =', deldeg, ' (deg)     sinp =', sinp
    endif

    return		; fin de l'initialisation au sens Mercier
    end			; fin de IN_MALAX_2D

;_____________________________________________________________________________

; appele'e par RH_DPATCHFITS_NRH  et  RH_DP_CALIB .

PRO INIT_MALAX_2D,  date,  hmer,  dec,  sref,  rot_p  ; entree.
	
; But : 1er maillon de la chaine d'initialisation du common MALAX.
;	La raison pour laquelle on utilise la cascade INIT_MALAX_2D -> IN_MALAX
;	est expliquee ci dessous.

; Rem : Si on veut utliser les variables du common MALAX avec le soft du 2*1D
;	(sans E0 ni NS45) il ne faut pas appeler IN_MALAX par IN_MALAX_2D.

; Modifications:
; 99 aug 26: ie0, ie1, ie2 (drapeaux d'utilisation des antrenne E0, E1, E2),
;	       utilises pour les calcul des canaux "minimaux" new et nns, sont
;	       passes en entree a partir de DPATCHFITS.
; 01 jul 11: ajout de ins45 dans la liste d'appel dans DPACHFITS, pour decider
;		si nns=64 ou 128 dans le common MALAX.
;    sep 10: definition de ie0, ie1, ie2, ins45 transferee de DPATCHFITS dans
;		  INIT_MALAX_2D. Les valeurs ci-dessous permettent l'utilisa-
;		  tion de toutes les extensions. Si elles manquent les images
;		  seront sur-echantillonnees mais il suffit que les procedures
;		  soient coherentes (voir commentaires dans POS_HELIO et 
;		  CALCPOS).
; 02 mar 27: ajout de rot_p en entree pour correction angle p (0 sans, 1 avec)
;    avr  3: mise au clair de commentaires sur new, nns, ins45 etc dans ce
;		fichier et dans CALCPOS. 

; Drapeaux d'utilisation des extensions
	; Ces drapeux ne servent en fait qu'a choisir les valeurs dew, dns,
	;    new, nns. On rappelle que new et nns ne sont pas forcement les
	;    puissances de 2 MINIMALES pour echantillonner correctement les
	;    images, mais les nbres de points choisis (surechantillonnage 
	;    eventuel). Il s'agit simplement d'etre coherent avec le npi=128
	;    choisi dans RH_DPNEW
	; Si on veut remplir le common MALAX pour une utilisation avec d'autres
	;    programmes (2 fois 1D, quand les images EW et NS etaient obtenues
	;    sans E2 et NS45) on peut ne pas mettre tous ces drapeaux a 1.
	; C'est la raison pour laquelle ces drapeaux sont definis ici, en amont
;	     de IN_MALAX, qui reste ainsi utilisable de facon plus generale.

    ie0   = 1L
    ie1   = 1L
    ie2   = 1L
    ins45 = 1L
	; Rem : on a ainsi toujours  new = nns = 128 dans le common MALAX.

; Gestion de sref
	; sref = 'SOLEIL' , '3C_405' ,  etc.
    isour = 2L
    if (sref  eq  'SOLEIL') then  isour = 1L

    date_0 = date		; pour eviter date(0) qui plante (nouvelle 
				;   version IDL ?).
    jour = long(date_0(0))
    mois = long(date_0(1))
    an   = long(date_0(2))
    icorion = 0L	; pas de correction ionospherique 
    icorpoi = 0L	; donnees de Poitiers non utilisee pour raffiner le
			;  modele ionospherique du CNET. 

    IN_MALAX_2D, $					; tout en entree.
	    ie0    , ie1    , ie2    , ins45,	$
	    isour  , rot_p  ,			$
	    hmer(0), hmer(1), hmer(2),		$	; TU pour soleil.
	    dec(0) , dec(1) , dec(2) , 		$
	    icorion, icorpoi, jour   , mois, an
    return
    end
